package mdp.rtdp;

import java.util.*;
import java.text.*;
import java.awt.Color;
import java.io.*;

public class BRTDP extends Solver {
	boolean found_goal = false;
	int goals = 0;

	public BRTDP(Racetrack r) {
		super(r);
	}

	public String getName() {
		return "BRTDP";
	}

	public double runTrial() {
				
		_racetrack.clearMarks();

		// Select start state at random
		Racetrack.State s = _racetrack.chooseInitialState();
		s.setMark();
		
		if (VISUALIZE_SEARCH) {
			System.out.println("Visiting first: " + s);
			_racetrack.bd.setCell(s.y, s.x, Color.cyan, null);
			_racetrack.bd.repaint();		
		}

		// Execute trajectory until max length reached or success
		// Note: assume no discounting (we have bounded total reward)
		double accum_reward = 0.0;
		Stack states = new Stack();
		for (int actions = 0; actions < Racetrack.MAX_TRAJ_LENGTH
				&& s.ctype != 'f'; actions++) {
			
			// Populate neighbors
			s.populateNeighbors();
			states.push(s);
			
			// Bellman update and get greedy action for upper bound
			Racetrack.BestAction act = s.doBellmanUpdate(true);
			
			// Take action and randomly pick next state
			//Racetrack.State temp_s = s.chooseNextState(act);
			Racetrack.State old_s = s;
			Racetrack.State new_s = s.chooseNextStateBoundGap(act.ub_action);
			
			if (new_s == null) {
				if (VISUALIZE_SEARCH) System.out.println("null next state: terminate trial");
				break;
			} else {
				s = new_s;
				s.setMark();
			}
			// Decrement reward for action taken
			accum_reward = accum_reward - 1.0;

			if (VISUALIZE_SEARCH && !new_s.equals(old_s)) {
				System.out.println("Visiting: " + new_s);
				_racetrack.restoreBackground(old_s);
				_racetrack.bd.setCell(new_s.y, new_s.x, Color.cyan, null);
				_racetrack.bd.repaint();
				try {
					Thread.currentThread().sleep(30);
				} catch (InterruptedException e) {
					System.out.println(e);
				}
			}

			
		}
		while (!states.empty()) {
			s = (Racetrack.State) states.pop();
			s.doBellmanUpdate(true);
		}
		return accum_reward;
	}
}
