package mdp.rtdp;

// TODO: -10 reward for hitting wall
// TODO: Put in all output diagnostics... backups, avg rewards over k trials, no display
// TODO: Experiment with different probability weightings of VPI
// TODO: Experiment with graphs of final probability, modifications to integral
// TODO: Experiment with trajectory cutoffs... 0d for now since myopic prematurely cuts
//       of search

import java.util.*;
import java.text.*;
import java.awt.Color;
import java.io.*;

public class VPIRTDP extends Solver {

	public VPIRTDP(Racetrack r) {
		super(r);
		r.MAINTAIN_QVALUES = true;
	}

	public String getName() {
		return "VPIRTDP";
	}
	
	public double runTrial() {
		
		// Bounds printing for print_state, set ps = null to disable		
		_racetrack.clearMarks();
		GOAL_FOUND = false;
		
		// Select start state at random
		Racetrack.State s = _racetrack.chooseInitialState();
		// Execute trajectory until max length reached or success
		// Note: assume no discounting (we have bounded total reward)
		s.setMark();
		
		if (VISUALIZE_SEARCH) {
			//System.out.println("Visiting first: " + s);
			_racetrack.bd.setCell(s.y, s.x, Color.cyan, null);
			_racetrack.bd.repaint();		
		}
		
		double accum_reward = 0.0;
		Stack states = new Stack();
		for (int actions = 0; actions < Racetrack.MAX_TRAJ_LENGTH
				&& s.ctype != 'f'; actions++) {
			
			// Populate neighbors
			s.populateNeighbors();
			states.push(s);
			
			// Bellman update and get greedy action for upper bound
			Racetrack.BestAction act = s.doBellmanUpdate(true);
			
			// Take action and randomly pick next state
			Racetrack.State old_s = s;
			Racetrack.State new_s = s.chooseNextStateVPI(act);
			
			// Note: many policies need to loop due to stochasticity, so
			//       should not do loop check here... can try in nextVPIState
			//       though... works better there.
			if (new_s == null /*|| new_s.isMarked()*/) {
				if (VISUALIZE_SEARCH) System.out.println("null next state: terminate trial");
				break;
			} else {
				s = new_s;
				s.setMark();
			}
			// Decrement reward for action taken
			accum_reward += s.getReward(act.avg_action);
			
			if (VISUALIZE_SEARCH && !new_s.equals(old_s)) {
				System.out.println("Visiting: " + new_s);
				_racetrack.restoreBackground(old_s);
				_racetrack.bd.setCell(new_s.y, new_s.x, Color.cyan, null);
				_racetrack.bd.repaint();
				try {
					Thread.currentThread().sleep(10);
				} catch (InterruptedException e) {
					System.out.println(e);
				}
			}

		}
		
		//if (GOAL_FOUND) System.out.println("Goal found!!! [" + states.size() + "] " + states);
		while (!states.empty()) {
			s = (Racetrack.State) states.pop();
			//if (GOAL_FOUND)
			//	System.out.print("Update: " + s);
			s.doBellmanUpdate(true);
			//if (GOAL_FOUND)
			//	System.out.println(" --> " + s);
		}
		
		return accum_reward;
	}
}
