package mdp.rtdp;

import java.util.*;
import java.text.*;

public class ValueIter extends Solver {
	
	public ValueIter(Racetrack r) {
		super(r);
		
		// Value iteration needs to know about all possible states
		for (int x = 0; x < r._track._width; x++)
			for (int y = 0; y < r._track._height; y++)
				if (r._track.get(x,y) != '@') 
					for (int dx = -10; dx < 10; dx++)
						for (int dy = -10; dy < 10; dy++)
							r.addStateAndPopulateNeighbors(x, y, dx, dy);
	}
		
	/////////////////////////////////////////////////////////////////////////////
	//                  One Synchronous Sweep of Value Iteration
	/////////////////////////////////////////////////////////////////////////////

	public String getName() {
		return "VI";
	}

	// Here we run one "trial", for value iteration, this is one full sychronous
	// sweep through Bellman updates for all states.  
	public double runTrial() {

		double accum_error = 0.0;
		
		// Iterate over all states
		Object[] states = _racetrack._states.keySet().toArray();
		for (int i = 0; i < states.length; i++) {
			
			Racetrack.State s = (Racetrack.State)states[i];
			if (s.ctype == 'f') continue; // Don't update goal states, value known!
			
			// Do Bellman update and track Bellman error for bookkeeping
			double value_before = s._valueUpper;
			
			s.doBellmanUpdate(true /* do update */);
			
			accum_error += Math.abs(value_before - s._valueUpper); 
		}

		// Return average Bellman error
		return accum_error / _racetrack._states.keySet().size();
	}
	
}
