package mdp.rtdp;

import java.util.*;
import java.text.*;

public class ValueIterReach extends Solver {
	
	public ValueIterReach(Racetrack r) {
		super(r);
		
		// Recursively add all reachable states to Racetrack state model
		for (Iterator i = _racetrack._startStates.iterator(); i.hasNext(); ) {
			addStates((Racetrack.State)i.next());
		}
	}
	
	public String getName() {
		return "VI-Reachable";
	}

	// Value iteration needs to build all reachable states first
	public void addStates(Racetrack.State s) {
		
		// Check if neighbors of s not populated... if not, populate and recurse
		if (!s.hasNeighbors()) {
			s.populateNeighbors();
			//System.out.println(s);
			
			// Now recursively call addStates for all neighbors
			for (int act = 0; act < 9; act++) 
				for (int index = 0; index < s._nextState[act].length; index++)
					addStates(s._nextState[act][index]);		
		}
	}
	
	/////////////////////////////////////////////////////////////////////////////
	//                  One Synchronous Sweep of Value Iteration
	/////////////////////////////////////////////////////////////////////////////
	
	// Here we run one "trial", for value iteration, this is one full sychronous
	// sweep through Bellman updates for all states.  
	public double runTrial() {

		double accum_error = 0.0;
		
		// Iterate over all states
		for (Iterator i = _racetrack._states.keySet().iterator(); i.hasNext(); ) {
			
			Racetrack.State s = (Racetrack.State)i.next();
			if (s.ctype == 'f') continue; // Don't update goal states, value known!
			
			// Do Bellman update and track Bellman error for bookkeeping
			double value_before = s._valueUpper;
			
			s.doBellmanUpdate(true /* do update */);
			
			accum_error += Math.abs(value_before - s._valueUpper); 
		}

		// Return average Bellman error
		return accum_error / _racetrack._states.keySet().size();
	}
	
}
