package mdp.rtdp;

import java.io.*;
import java.text.DecimalFormat;
import java.util.*;

// Converts racetrack diagram to a .fig file
// Use fig2dev to convert to .eps, etc...
public class genPolPerf {

    public static final String PREFIX = "mdp/rtdp/";
    public static final String RACETRACK = "block";
    public static final String SUFFIX = ".txt";
    public static int[] SIZES = new int[] { 10, 20, 30, 40, 50, 60, 70, 80, 90, 100 };
    public static String[] ALGS = new String[] { "RTDP", "BRTDP", "FRTDP", "VPIRTDP" };
    
	public static DecimalFormat _df = new DecimalFormat("#.##");

    public static void main(String[] args) {
	
		BufferedReader in = null;
		PrintStream ps_time  = null;
		PrintStream ps_space = null;
		try {
		    ps_time = new PrintStream(new FileOutputStream(
		    		PREFIX + RACETRACK + "_" + ALGS.length + "_" + SIZES.length + "_time.txt"));
		    ps_space = new PrintStream(new FileOutputStream(
		    		PREFIX + RACETRACK + "_" + ALGS.length + "_" + SIZES.length + "_space.txt"));
	
	    	for (int sz = 0; sz < SIZES.length; sz++) {
	    		
	    		ps_time.print(SIZES[sz]);
	    		ps_space.print(SIZES[sz]);
	    		for (int alg = 0; alg < ALGS.length; alg++) {
	    			
	    			String input_file = PREFIX + ALGS[alg] + 
						"_" + RACETRACK + SIZES[sz] + SUFFIX;
	    			
	    			System.out.println("Processing: " + input_file);
	    					    		
		    		in = new BufferedReader(new FileReader(input_file));
		    		ps_time.print("\t" + _df.format(findFirst(in, true)));
		    		in.close();
		    		in = new BufferedReader(new FileReader(input_file));
		    		ps_space.print("\t" + _df.format(findFirst(in, false)));
		    		in.close();
		    	}
	    		ps_time.println();
	    		ps_space.println();
		    }
	
	    	ps_time.close();
	    	ps_space.close();
		}
		catch (Exception ignore) {
		    System.out.println("ERROR: " + ignore);
		    ignore.printStackTrace();
		    System.exit(1);
		}
	}
    
    public static double findFirst(BufferedReader r, boolean time) 
    	throws Exception {
    	
    	double max = 0d;
    	String line;
    	while (r != null && (line = r.readLine()) != null) {
    		//System.out.println(line);
    		String[] split = line.split("\t");
    		//System.out.println(split[1] + ", " + split[4] + ", " + split[6]);
    		max = time ? Double.parseDouble(split[6]) : Double.parseDouble(split[4]);
    		if (Double.parseDouble(split[1]) > -100d) {
    			return max;
    		}
    	}
    	return max;
    }
    	
}
